package org.example;

import org.graphstream.algorithm.Toolkit;
import org.graphstream.algorithm.generator.BarabasiAlbertGenerator;
import org.graphstream.algorithm.generator.Generator;
import org.graphstream.algorithm.generator.RandomGenerator;
import org.graphstream.graph.BreadthFirstIterator;
import org.graphstream.graph.Edge;
import org.graphstream.graph.Graph;
import org.graphstream.graph.Node;
import org.graphstream.graph.implementations.DefaultGraph;
import org.graphstream.graph.implementations.SingleGraph;
import org.graphstream.stream.file.FileSource;
import org.graphstream.stream.file.FileSourceEdge;

import java.io.*;
import java.util.Arrays;
import java.util.OptionalDouble;

import static org.graphstream.algorithm.Toolkit.*;

public class propagation {
    public static double seuilEpidemiqueReseauAleatoire(double degreMoyen, int nbNoeud){
        System.out.println("Réseau aleatoire :");
        Graph graphe = new SingleGraph("Random");
        Generator gen = new RandomGenerator(degreMoyen);
        gen.addSink(graphe);
        gen.begin();
        for (int i = 0; i < nbNoeud; i++)
            gen.nextEvents();
        gen.end();
        double seuilEpidemique = 1 / (degreMoyen + 1);
        System.out.println("le seuil epidemique est : " + seuilEpidemique);
        return seuilEpidemique;
    }
    public static double seuilEpidemique(Graph graph){
        System.out.println("Notre réseau initiale :");
        double degreMoyen = (float) averageDegree(graph);
        System.out.println("le degre moyen du graphe est : "+ degreMoyen);
        double degreMoyenCarre = moyenneDegreCarre(graph);
        System.out.println("le degre moyen carre du graphe est : " + degreMoyenCarre);
        double seuilEpidemique = degreMoyen / degreMoyenCarre;
        System.out.println("le seuil epidemique est : " + seuilEpidemique);
        return seuilEpidemique;

    }
    public static double moyenneDegreCarre(Graph graph){
        double somme=0;
        double nbrNoeuds = graph.getNodeCount();
        for(Node n : graph) {
            somme += n.getDegree() * n.getDegree();
        }
        return somme / nbrNoeuds;
    }
    public static int estInfecte(Node node, int nbrInfection){
        double proba = Math.random();
        double envoieMail = 1.0/7.0;
        if(proba < envoieMail  && !node.hasAttribute("immunise")){
            if(!node.hasAttribute("infecte")){
                node.setAttribute("infecte", true);
                nbrInfection++;
            }
        }
        return nbrInfection;
    }

    public static int estImmunise(Node node, int nbrInfection){
        double proba = Math.random();
        double antiVirus = 1.0/14.0;
        if(proba < antiVirus){
            if(node.hasAttribute("infecte")){
                node.removeAttribute("infecte");
                nbrInfection--;
            }
        }
        return nbrInfection;
    }

    public static void ecrireDataFichier(StringBuilder data,int nbrSce) throws IOException {
        File fichier = new File("./scenarios/scenarioBarbasi"+nbrSce+".dat");
        if (!fichier.exists()) {
            fichier.createNewFile();
        }
        FileWriter fw = new FileWriter(fichier.getAbsoluteFile(),false);
        BufferedWriter bw = new BufferedWriter(fw);
        bw.write(data.toString());
        bw.close();
    }

    public static void propagationScenario1(Graph graph) throws IOException {
        Node s = graph.getNode(0);
        s.setAttribute("infecte", true);
        int nbrInfect = 1;
        StringBuilder data = new StringBuilder();
        for(int i = 0; i<84; i++){
            for (Node n : graph){
                if(n.hasAttribute("infecte")){
                    for(Edge e : n){
                        nbrInfect = estInfecte(e.getOpposite(n), nbrInfect);
                    }
                }
                nbrInfect = estImmunise(n, nbrInfect);
            }
            data.append(i+1).append(" ").append(nbrInfect).append(System.getProperty("line.separator"));
        }
        ecrireDataFichier(data,1);
    }
    public static void propagationScenario2(Graph graph) throws IOException {
        int nbrImmunise = 0;
        double proba = Math.random();
        double antiVirus = 1.0/2.0;
        StringBuilder data = new StringBuilder();
        for(Node n : graph) {
            if ((int) (Math.random() * 2 + 1) == 1) {
                n.setAttribute("immunise", true);
                nbrImmunise += 1;
            }
        }
        Node s = graph.getNode(0);
        s.setAttribute("infecte", true);
        int nbInfect = 1;
        for(int i = 0; i<84; i++){
            for (Node n : graph){
                if(n.hasAttribute("infecte")){
                    for(Edge e : n){
                        nbInfect = estInfecte(e.getOpposite(n), nbInfect);
                    }
                }
                nbInfect = estImmunise(n, nbInfect);
            }
            data.append(i+1).append(" ").append(nbInfect).append(System.getProperty("line.separator"));
        }
        ecrireDataFichier(data,2);
        double degreMoyen = 0.0;
        double degreMoyenCarre;
        double somme=0;
        for(Node n : graph) {
            if(!n.hasAttribute("immunise")) {
                degreMoyen += n.getDegree();
                somme += n.getDegree() * n.getDegree();
            }
        }

        int nbrNoeuds = graph.getNodeCount() - nbrImmunise;
        degreMoyen /= nbrNoeuds;
        degreMoyenCarre = somme / nbrNoeuds;

        System.out.println("Le degre moyen du graphe modifié est : "+ degreMoyen);
        System.out.println("Le degre moyen carre du graphe est : " + degreMoyenCarre);
        System.out.println("Seuil épidémique du réseau modifié est : " + (degreMoyen / degreMoyenCarre));
    }

    public static void propagationScenario3(Graph graph) throws IOException {
        int nbrImmunise = 0;
        double proba = Math.random();
        double antiVirus = 1.0/2.0;
        StringBuilder data = new StringBuilder();
        double degreMoyenG0 = 0;
        double degreMoyenG1 = 0;
        for(Node n : graph) {
            if ((int) (Math.random() * 2 + 1) == 1 && n.edges().count() != 0) {
                int voisin = (int)(Math.random() * n.edges().count() );
                n.getEdge(voisin).getOpposite(n).setAttribute("immunise", true);
                nbrImmunise += 1;
                degreMoyenG0 += n.getDegree();
                degreMoyenG1 += n.getEdge(voisin).getOpposite(n).getDegree();
            }
        }
        Node s = graph.getNode(0);
        s.setAttribute("infecte", true);
        int nbrInfect = 1;
        for(int i = 0; i<84; i++){
            for (Node n : graph){
                if(n.hasAttribute("infecte")){
                    for(Edge e : n){
                        nbrInfect = estInfecte(e.getOpposite(n), nbrInfect);
                    }
                }
                nbrInfect = estImmunise(n, nbrInfect);
            }
            data.append(i+1).append(" ").append(nbrInfect).append(System.getProperty("line.separator"));

        }
        ecrireDataFichier(data,3);

        double degreMoyen = 0.0;
        double degreMoyenCarre;
        double somme=0;
        for(Node n : graph) {
            if(!n.hasAttribute("immunise")) {
                degreMoyen += n.getDegree();
                somme += n.getDegree() * n.getDegree();
            }
        }

        degreMoyenG0 = degreMoyenG0/nbrImmunise;
        degreMoyenG1 = degreMoyenG1/nbrImmunise;

        System.out.println("Le degre moyen du groupe 0 : " + degreMoyenG0);
        System.out.println("Le degre moyen du groupe 1 : " + degreMoyenG1);

        int nbrNoeuds = graph.getNodeCount() - nbrImmunise;

        degreMoyen /= nbrNoeuds;
        degreMoyenCarre = somme / nbrNoeuds;

        System.out.println("Le degre moyen du graphe modifié est : "+ degreMoyen);
        System.out.println("Le degre moyen carre du graphe est : " + degreMoyenCarre);
        System.out.println("Seuil épidémique du réseau modifié est : " + (degreMoyen / degreMoyenCarre));
    }

    public static Graph reseauAleatoire(double degreMoyen, int nbNoeud){
        System.out.println("Réseau avec la méthode aleatoire : ");
        Graph graphe = new SingleGraph("Random");
        Generator gen = new RandomGenerator(degreMoyen);
        gen.addSink(graphe);
        gen.begin();
        for (int i = 0; i < nbNoeud; i++)
            gen.nextEvents();
        gen.end();
        return graphe;
    }
    public static Graph reseauBarbasi(float degreMoyen, int nbNoeud){
        System.out.println("Réseau avec la méthode d'attachement préférentiel (Barabasi-Albert)  : ");
        Graph graphe = new SingleGraph("Barabàsi-Albert");
        // Between 1 and 3 new links per node added.
        Generator gen = new BarabasiAlbertGenerator((int)(degreMoyen));
        // Generate 100 nodes:
        gen.addSink(graphe);
        gen.begin();
        for(int i=0; i<nbNoeud; i++) {
            gen.nextEvents();
        }
        gen.end();
        return graphe;
    }



    public static void main(String[] args) throws IOException {
        String filePath = "com-dblp.ungraph.txt";
        Graph g = new DefaultGraph("g");
        FileSource fs = new FileSourceEdge();
        fs.addSink(g);
        System.setProperty("org.graphstream.ui", "swing");
        try {
            fs.readAll(filePath);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            fs.removeSink(g);
        }

        int nbNoeuds = g.getNodeCount();
        System.out.println("le nombre de noeuds du graphe est : "+ nbNoeuds);
        int nbEdges = g.getEdgeCount();
        System.out.println("le nombre de liens du graphe est : "+ nbEdges);
        double degreMoyen = averageDegree(g);
        System.out.println("le degre moyen du graphe est : "+ degreMoyen);
        double B = 0.1428;
        System.out.println("la probabilité de trassmission : "+ B);
        double U = 0.0714;
        System.out.println("la probabilité d'etre remis : "+ U);
        double landa = B/U;
        System.out.println("le taux de propagation du virus : "+ landa);

        seuilEpidemique(g);

        //System.out.println("Notre réseau modifie:");
        //propagationScenario1(g);
        //propagationScenario2(g);
        //propagationScenario3(g);

        //Graph g2 = reseauAleatoire(6.622,317080);
        //propagationScenario1(g2);
        //propagationScenario2(g2);
        //propagationScenario3(g2);

        Graph Barbasi = reseauBarbasi((float)6.622,317080);
        //propagationScenario1(Barbasi);
        //propagationScenario2(Barbasi);
        propagationScenario3(Barbasi);


    }

}
